@@ -9,29 +9,27 @@
 // or else upgrading old project file will not work
 // reliably.
 // TODO QGIS 4: remove in favor of _QGIS_VERSION
-#define VERSION "3.36.3-Maidenhead"
-#define _QGIS_VERSION "3.36.3-Maidenhead"
+#define VERSION "3.38.0-Grenoble"
+#define _QGIS_VERSION "3.38.0-Grenoble"
 
 //used in vim src/core/qgis.cpp
 //The way below should work but it resolves to a number like 0110 which the compiler treats as octal I think
 //because debuggin it out shows the decimal number 72 which results in incorrect version status.
 //As a short term fix I (Tim) am defining the version in top level cmake. It would be good to
 //reinstate this more generic approach below at some point though
-//#define VERSION_INT 3363
+//#define VERSION_INT 3380
 // TODO QGIS 4: Remove in favor of _QGIS_VERSION_INT
-#define VERSION_INT 33603
-#define _QGIS_VERSION_INT 33603
-#define ABISYM(x) x ## 33603
+#define VERSION_INT 33800
+#define _QGIS_VERSION_INT 33800
+#define ABISYM(x) x ## 33800
 //used in main.cpp and anywhere else where the release name is needed
-#define RELEASE_NAME "Maidenhead"
+#define RELEASE_NAME "Grenoble"
 
 #define QGIS_PLUGIN_SUBDIR "lib/qgis/plugins"
 #define QGIS_DATA_SUBDIR "share/qgis"
 #define QGIS_LIBEXEC_SUBDIR "lib/qgis"
 #define QGIS_LIB_SUBDIR "lib"
 #define QGIS_QML_SUBDIR "qml"
-#define CMAKE_INSTALL_PREFIX "/usr"
-#define CMAKE_SOURCE_DIR "/build/qgis/src/qgis-3.36.3"
 
 #define QGIS_SERVER_MODULE_SUBDIR "lib/qgis/server"
 
@@ -44,16 +42,14 @@
 #define QGIS_MACAPP_BUNDLE 
 #endif
 
-#define QT_PLUGINS_DIR "/usr/lib/qt/plugins"
-
-#define PYTHON_VERSION "3.12.3"
+#define PYTHON_VERSION "3.12.4"
 #define PYTHON_VERSION_MAJOR "3"
 #define PYTHON_VERSION_MINOR "12"
-#define PYTHON_VERSION_PATCH "3"
+#define PYTHON_VERSION_PATCH "4"
 
 #define PROJ_VERSION_MAJOR 9
 #define PROJ_VERSION_MINOR 4
-#define PROJ_VERSION_PATCH 0
+#define PROJ_VERSION_PATCH 1
 
 /* #undef USING_NMAKE */
 
