@@ -53,7 +53,6 @@ class QgsMapSettings;
 /**
  * \brief Handles asynchronous download of images
  * \ingroup core
- * \since QGIS 2.8
  */
 class CORE_EXPORT QgsImageFetcher : public QObject
 {
@@ -92,25 +91,6 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
 
   public:
 
-    /**
-     * Enumeration with capabilities that raster providers might implement.
-     * \since QGIS 3.0
-     */
-    enum ProviderCapability
-    {
-      NoProviderCapabilities = 0,       //!< Provider has no capabilities
-      ReadLayerMetadata = 1 << 1, //!< Provider can read layer metadata from data store. Since QGIS 3.0. See QgsDataProvider::layerMetadata()
-      WriteLayerMetadata = 1 << 2, //!< Provider can write layer metadata to the data store. Since QGIS 3.0. See QgsDataProvider::writeLayerMetadata()
-      ProviderHintBenefitsFromResampling = 1 << 3, //!< Provider benefits from resampling and should apply user default resampling settings (since QGIS 3.10)
-      ProviderHintCanPerformProviderResampling = 1 << 4, //!< Provider can perform resampling (to be opposed to post rendering resampling) (since QGIS 3.16)
-      ReloadData = 1 << 5, //!< Is able to force reload data / clear local caches. Since QGIS 3.18, see QgsDataProvider::reloadProviderData()
-      DpiDependentData = 1 << 6, //! Provider's rendering is dependent on requested pixel size of the viewport (since QGIS 3.20)
-      NativeRasterAttributeTable = 1 << 7, //!< Indicates that the provider supports native raster attribute table (since QGIS 3.30)
-    };
-
-    //! Provider capabilities
-    Q_DECLARE_FLAGS( ProviderCapabilities, ProviderCapability )
-
     QgsRasterDataProvider();
 
     /**
@@ -129,9 +109,8 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
 
     /**
      * Returns flags containing the supported capabilities of the data provider.
-     * \since QGIS 3.0
      */
-    virtual QgsRasterDataProvider::ProviderCapabilities providerCapabilities() const;
+    virtual Qgis::RasterProviderCapabilities providerCapabilities() const;
 
     /* It makes no sense to set input on provider */
     bool setInput( QgsRasterInterface *input ) override { Q_UNUSED( input ) return false; }
@@ -229,13 +208,11 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
 
     /**
      * Read band scale for raster value
-     * \since QGIS 2.3
      */
     virtual double bandScale( int bandNo ) const { Q_UNUSED( bandNo ) return 1.0; }
 
     /**
      * Read band offset for raster value
-     * \since QGIS 2.3
      */
     virtual double bandOffset( int bandNo ) const { Q_UNUSED( bandNo ) return 0.0; }
 
@@ -311,7 +288,6 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      *         to caller.
      *
      *
-     * \since QGIS 2.8
      */
     virtual QgsImageFetcher *getLegendGraphicFetcher( const QgsMapSettings *mapSettings ) SIP_FACTORY
     {
@@ -381,12 +357,6 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      */
     bool hasPyramids();
 
-    /**
-     * Returns metadata in a format suitable for feeding directly
-     * into a subset of the GUI raster properties "Metadata" tab.
-     */
-    virtual QString htmlMetadata() = 0;
-
     /**
      * Identify raster value(s) found on the point position. The context
      * parameters extent, width and height are important to identify
@@ -471,7 +441,6 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      * Checks whether the provider is in editing mode, i.e. raster write operations will be accepted.
      * By default providers are not editable. Use setEditable() method to enable/disable editing.
      * \see setEditable(), writeBlock()
-     * \since QGIS 3.0
      */
     virtual bool isEditable() const { return false; }
 
@@ -483,14 +452,13 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      * the underlying data source into editing mode, so it is necessary to check the return
      * value whether the operation was successful.
      * \see isEditable(), writeBlock()
-     * \since QGIS 3.0
      */
     virtual bool setEditable( bool enabled ) { Q_UNUSED( enabled ) return false; }
 
     // TODO: add data type (may be different from band type)
 
     //! Writes into the provider datasource
-    virtual bool write( void *data, int band, int width, int height, int xOffset, int yOffset )
+    virtual bool write( const void *data, int band, int width, int height, int xOffset, int yOffset )
     {
       Q_UNUSED( data )
       Q_UNUSED( band )
@@ -515,7 +483,6 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      * in order to allow write operations.
      * \see isEditable(), setEditable()
      * \returns TRUE on success
-     * \since QGIS 3.0
      */
     bool writeBlock( QgsRasterBlock *block, int band, int xOffset = 0, int yOffset = 0 );
 
@@ -624,19 +591,17 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
     /**
      * Converts a raster identify \a format to a capability.
      */
-    static Capability identifyFormatToCapability( Qgis::RasterIdentifyFormat format );
+    static Qgis::RasterInterfaceCapability identifyFormatToCapability( Qgis::RasterIdentifyFormat format );
 
     /**
      * Step width for raster iterations.
      * \see stepHeight()
-     * \since QGIS 3.0
      */
     virtual int stepWidth() const { return QgsRasterIterator::DEFAULT_MAXIMUM_TILE_WIDTH; }
 
     /**
      * Step height for raster iterations.
      * \see stepWidth()
-     * \since QGIS 3.0
      */
     virtual int stepHeight() const { return QgsRasterIterator::DEFAULT_MAXIMUM_TILE_HEIGHT; }
 
@@ -844,7 +809,6 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
 
     /**
      * Emit a message to be displayed on status bar, usually used by network providers (WMS,WCS)
-     * \since QGIS 2.14
      */
     void statusChanged( const QString & ) const;
 
@@ -931,8 +895,6 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
 
 };
 
-Q_DECLARE_OPERATORS_FOR_FLAGS( QgsRasterDataProvider::ProviderCapabilities )
-
 // clazy:excludeall=qstring-allocations
 
 #endif
